# This is free and unencumbered software released into the public domain.

"""ASIMOV for LangChain."""

from ._version import __version__, __version_tuple__
from .document_loaders import AsimovLoader
from .errors import AsimovModuleNotFound

__all__ = [
    'AsimovLoader',
    'AsimovModuleNotFound',
    '__version__',
    '__version_tuple__',
]



# This is free and unencumbered software released into the public domain.

"""ASIMOV for LangChain: Document loaders."""

from __future__ import annotations # for Python 3.9

import json
import logging
import subprocess
from .errors import AsimovModuleNotFound
from langchain_core.document_loaders.base import BaseLoader
from langchain_core.documents import Document
from pyld import jsonld
from typing import Any, Iterator, cast, override

logger = logging.getLogger(__file__)

JSONLD_CONTEXT = {
    "@version": 1.1,
    "know": "https://know.dev/",
    "xsd": "http://www.w3.org/2001/XMLSchema#",
}

class AsimovLoader(BaseLoader):
    """
    ASIMOV document loader integration.

    Setup:
        Install ``langchain-asimov``:

        ```bash
        pip install -U langchain-asimov
        ```

    Instantiate:
        ```python
        from langchain_asimov import AsimovLoader

        loader = AsimovLoader(
            module="serpapi",
            url="https://duckduckgo.com/?q=Isaac+Asimov"
        )
        ```
    """
    def __init__(self, module: str, url: str, **kwargs: Any) -> None:
        self.module = module
        self.url = url

    @override
    def lazy_load(self) -> Iterator[Document]:
        try:
            result = subprocess.run(
                [f"asimov-{self.module}-importer", self.url],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
            )
            result.check_returncode()
            output = json.loads(result.stdout)
            output = cast(dict, jsonld.flatten(output, JSONLD_CONTEXT))
            for resource in output["@graph"]:
                resource_id = resource["@id"]
                page_content = describe(resource)
                yield Document(page_content, id=resource_id, metadata=resource)
        except FileNotFoundError as error:
            #logger.exception(error)
            raise AsimovModuleNotFound(self.module) from (error if __debug__ else None)
        except subprocess.CalledProcessError as error:
            #logger.exception(error)
            raise error # TODO
        except json.decoder.JSONDecodeError as error:
            #logger.exception(error)
            raise error # TODO
        except jsonld.JsonLdError as error:
            #logger.exception(error)
            raise error # TODO

def describe(resource: dict) -> str:
    return resource["know:summary"]["@value"] # TODO



# This is free and unencumbered software released into the public domain.

"""ASIMOV for LangChain: Exception classes."""

from __future__ import annotations # for Python 3.9

class AsimovModuleNotFound(Exception):
    """Exception raised when a module cannot be found or imported.

    Attributes:
        module_name: The name of the module that was not found
        message: Explanation of the error
    """

    def __init__(self, module_name: str, message: str | None = None) -> None:
        """Initializes the `AsimovModuleNotFound` exception.

        Args:
            module_name: The name of the module that was not found
            message: Optional custom error message. If not provided,
                    a default message will be generated.
        """
        self.module_name = module_name
        if message is None:
            message = f"Module '{module_name}' not found"
        self.message = message
        super().__init__(self.message)

    def __str__(self) -> str:
        """Returns a string representation of the exception."""
        return self.message

    def __repr__(self) -> str:
        """Returns a detailed string representation of the exception."""
        return f"{self.__class__.__name__}(module_name={self.module_name!r}, message={self.message!r})"
